pro simprior, tau=tau, intdisp=intdisp, nobs=nobs, measerr=measerr $
                 , priortau=priortau, allowneg=allowneg $
                 , ntrials=ntrials, noplot=noplot $
                 , minext=minext, maxext=maxext, dext=dext $
                 , sigma=sigma, priorsigma=priorsigma $
                 , fitintdisp=fitintdisp, seed=seed $
                 , savefile=savefile, rvobs=rvobs


; simulates an extinction distribution and tries to recover it
;
; We simulate <ntrials> supernovae.
;
; We assume each SN has an intrinsic color which is a Gaussian
; variable with mean 0 and std-dev <intdisp>; the SN keeps the 
; same intrinisic color in all observations.
;
; Added to the intrinsic color is a true extinction, which can
; either be an exponential with scale length <tau> or a 
; one-sided Gaussian with std-dev <sigma>. The extinction is
; assumed to be 3.3 times the color excess (unless you change rvobs),
; so it's like the colors are B-V and an extinction law of R_V = 3.1
;
; We simulate <nobs> observations of the SN color, adding a
; Gaussian measurement error with std-dev <measerr> for each
; observation.
;
; Then we fit the observations to determine the extinction.
; In the fit, we assume the intrinsic color dispersion is <fitintdisp>
; We use a prior in the fit, again either an exponential with
; scale length <priortau> or a Gaussian with std-dev <priorsigma>.
;
; If <priortau> or <priorsigma> are negative, we use a flat prior instead.
;
; If /allowneg is set, we allow fits with negative extinction.
; 
; set savefile = 'somefile.sav' to save your results
;
;
; Defaults:
;    ntrials = 10000
;
;    intdisp = 0.05 mag  (intrinsic dispersion in color)
;        tau = 0.5 mag   (we use the exponential distribution by default)
;
;       nobs = 5
;    measerr = 0.1 mag
;
;      rvobs = 3.3
;
;  fitindisp = intdisp   (we use the correct covariance matrix by default)
;   priortau = tau       (we use the correct prior by default)
; priorsigma = sigma     (                "                  )
;
;

if n_elements(tau) eq 1 and n_elements(sigma) eq 1 then $
  message,'Cannot set both tau (exponential) and sigma (gaussian) extinction'

trugauss = (n_elements(sigma) eq 1) ? 1 : 0
if (not trugauss) and (n_elements(tau) ne 1) then tau=0.5d

if n_elements(priortau) eq 1 and n_elements(priorsigma) eq 1 then $
  message,'Cannot set both tau (exponential) and sigma (gaussian) prior'

if n_elements(priortau) eq 1 then begin
    prigauss = 0
endif else if n_elements(priorsigma) eq 1 then begin
    prigauss = 1 
endif else begin             ; neither priortau nor priorsigma are set
    prigauss = trugauss
    if prigauss then priorsigma=sigma else priortau=tau
endelse

if n_elements(intdisp) ne 1 then intdisp=0.05d
if n_elements(fitintdisp) ne 1 then fitintdisp=intdisp
if n_elements(nobs) ne 1 then nobs=5
if n_elements(measerr) ne 1 then measerr=0.1d
allowneg = keyword_set(allowneg)

if n_elements(ntrials) ne 1 then ntrials = 10000l
plot = not keyword_set(noplot)

if n_elements(rvobs) ne 1 then rvobs=3.3d

if n_elements(minext) ne 1 then minext = -5.0d
if n_elements(maxext) ne 1 then maxext = 10.0d
if n_elements(dext) ne 1 then dext = 0.01d


truex = dblarr(ntrials)
meanx = dblarr(ntrials)
modex = dblarr(ntrials)

extinct = vec(minext,maxext,dext,nelems=next)

if prigauss then begin
    penalty = (priorsigma ge 0d) ? (extinct/priorsigma)^2 : dblarr(next)
endif else begin
    penalty = (priortau ge 0d) ? 2.0d * abs(extinct)/priortau : dblarr(next)
endelse

if not allowneg then penalty[where(extinct lt 0d)] = 1d9

; the error matrix has two parts:
;   diagonal terms include the (uncorrelated) measurement error
cov = diag_matrix(dblarr(nobs)+measerr^2)
;   all terms include the perfectly correlated intrinsic dispersion
;   (since the intrinsic color is the same for all obs of one SN)
cov += fitintdisp^2

cinv = invert(cov)
sumcinv = total(cinv[0,*])

for jj=0l,ntrials-1 do begin

    colors=dblarr(nobs)        ;array of color observations

    intrin=intdisp*randomn(seed)
    colors += intrin ;add a gaussian intrinsic disperison


    if trugauss then begin
        tru_ext = sigma*abs(randomn(seed))
    endif else begin
        ;generate exponential deviate prob = 1/tau exp(-extinct/tau)
        tru_ext=(-tau)*alog(randomu(seed))
    endelse 
    colors += tru_ext/rvobs   ; add color excess = extinction / rvobs

    colors += measerr*randomn(seed,nobs) ;add a gaussian error


    ; this is the calculation of chisq that we want:
    ;
    ; chi = dblarr(next)
    ; for kk=0,next-1 do begin
    ;     resid = colors-(extinct[kk]/rvobs)
    ;     chi[kk] = transpose(resid) # cinv # resid
    ; endfor
    ;
    ; because of the form of our inverse covariance matrix
    ; (one value on the diagonal, another value for all other elements)
    ; here we can do the same calculation, but way faster:
    sumcol = total(colors)
    aca = transpose(colors) # cinv # colors
    aca = aca[0]   ; turn it from an array to a number
    chi = aca - (extinct/rvobs)*sumcinv*sumcol*2.0d + nobs*sumcinv*(extinct/rvobs)^2

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;; add prior information as a log-likelihood penalty,normalization?
    chi += penalty
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    bestchi=min(chi,kk)
    modex[jj] = extinct[kk]

    probchi=exp(-(chi-bestchi)/2.0d)
    meanx[jj]=total(probchi*extinct)/total(probchi)

    truex[jj] = tru_ext

endfor                          ;jj 

diffarr = meanx-truex
diffmode = modex-truex


if plot then begin

    oldp = !p
    oldx = !x
    oldy = !y

    !p.multi = [0,2,2]

    dmh = myhist(diffarr,dmx,nbin=((ntrials/20.0d) < 50))
    ddh = myhist(diffmode,ddx,nbin=((ntrials/20.0d) < 50))

    !y.range = [0,max([dmh,ddh])]

    plot,dmx,dmh,psym=10,xtitle='p(X) Mean - true extinction' $
      ,color=rgb(255,255,255),ytitle='number'

    ds = 0.24
    
    junk = moment(diffarr,sd=sdmean)
    meanmean = junk[0]
    medmean = median(diffarr)
    print,'p(X) Mean:     mean: ',string(meanmean,format='(f+0.5)') + $
      '  median: ',string(medmean,format='(f+0.5)') + $
      '  sd: ',string(sdmean,format='(f0.5)')
    oplot,[meanmean,meanmean],!y.crange,linestyle=2,color=rgb(255,255,255)
    xyouts,total(!x.crange)*0.5d,(ds+1.0)*!y.crange[0]-ds*!y.crange[1], $
      'mean bias: ' + string(meanmean,format='(f+0.5)') + $
      '    !7r!3: ' + string(sdmean,format='(f0.5)'), $
      align=0.5,color=rgb(255,255,255)

    plot,ddx,ddh,psym=10,xtitle='p(X) Mode - true extinction' $
      ,color=rgb(255,255,255),ytitle='number'

    junk = moment(diffmode,sd=sdmode)
    meanmode = junk[0]
    medmode = median(diffmode)
    print,'p(X) Mode:     mean: ',string(meanmode,format='(f+0.5)') + $
      '  median: ',string(medmode,format='(f+0.5)') + $
      '  sd: ',string(sdmode,format='(f0.5)')
    oplot,[meanmode,meanmode],!y.crange,linestyle=2,color=rgb(255,255,255)
    xyouts,total(!x.crange)*0.5d,(ds+1.0)*!y.crange[0]-ds*!y.crange[1], $
      'mean bias: ' + string(meanmode,format='(f+0.5)') + $
      '    !7r!3: ' + string(sdmode,format='(f0.5)'), $
      align=0.5,color=rgb(255,255,255)

    nb = ((ntrials/20.0d) < 50)
    xmin = min([truex,meanx,modex])
    xmax = max([truex,meanx,modex])
    th = myhist(truex,tx,nbin=nb,min=xmin,max=xmax)
    mh = myhist(meanx,mx,nbin=nb,min=xmin,max=xmax)
    dh = myhist(modex,dx,nbin=nb,min=xmin,max=xmax)

    !y.range = [0,max([th,mh,dh])]

    plot,[tx,mx],[th,mh],/nodata $
      ,xtitle='extinction: true (white) and p(X) mean (red)' $
      ,color=rgb(255,255,255),ytitle='number'
    oplot,tx,th,psym=10,color=rgb(255,255,255)
    oplot,mx,mh,psym=10,color=rgb(255,0,0),linestyle=2

    plot,[tx,dx],[th,dh],/nodata $
      ,xtitle='extinction: true (white) and p(X) mode (red)' $
      ,color=rgb(255,255,255),ytitle='number'
    oplot,tx,th,psym=10,color=rgb(255,255,255)
    oplot,dx,dh,psym=10,color=rgb(255,0,0),linestyle=2


    !p = oldp
    !x = oldx
    !y = oldy

endif

if n_elements(savefile) eq 1 then save,file=savefile

end
